/*
 * steghide 0.5.1 - a steganography program
 * Copyright (C) 1999-2003 Stefan Hetzl <shetzl@chello.at>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * 2020:
 *  - Changed steghide branding to stegseek.
 *
 */

#include <iostream>

#include "SteghideError.h"
#include "colors.h"
#include "common.h"

SteghideError::SteghideError(void)
    : MessageBase(_("error, exiting. (no error message defined).")) {}

SteghideError::SteghideError(std::string msg) : MessageBase(msg) {}

SteghideError::SteghideError(const char *msgfmt, ...) : MessageBase() {
    va_list ap;
    va_start(ap, msgfmt);
    setMessage(vcompose(msgfmt, ap));
    va_end(ap);
}

void SteghideError::printMessage() const {
    // print a prefix unless we are in accesible mode
    if (!Args.Accessible.getValue()) {
        std::cerr << "[" << CRED("!") << "] ";
    }
    std::cerr << "error: " << getMessage() << std::endl;
}
